@@ -21,9 +21,7 @@
 
 
 #include "sysincludes.h"
-#include "msdos.h"
 #include "mtools.h"
-#include "vfat.h"
 #include "mainloop.h"
 #include "plain_io.h"
 #include "nameclash.h"
@@ -54,7 +52,7 @@ static int renameit(dos_name_t *dosname,
 		    direntry_t *targetEntry)
 {
 	Arg_t *arg = (Arg_t *) arg0;
-	int fat;
+	uint32_t fat;
 
 	targetEntry->dir = arg->entry->dir;
 	dosnameToDirentry(dosname, &targetEntry->dir);
@@ -65,7 +63,7 @@ static int renameit(dos_name_t *dosname,
 		/* get old direntry. It is important that we do this
 		 * on the actual direntry which is stored in the file,
 		 * and not on a copy, because we will modify it, and the
-		 * modification should be visible at file 
+		 * modification should be visible at file
 		 * de-allocation time */
 		movedEntry = getDirentry(arg->mp.File);
 		if(movedEntry->Dir != targetEntry->Dir) {
@@ -73,11 +71,11 @@ static int renameit(dos_name_t *dosname,
 			direntry_t subEntry;
 			Stream_t *oldDir;
 			/* we have a directory here. Change its parent link */
-			
+
 			initializeDirentry(&subEntry, arg->mp.File);
 
 			switch(vfat_lookup(&subEntry, "..", 2, ACCEPT_DIR,
-					   NULL, NULL)) {
+					   NULL, 0, NULL, 0)) {
 			    case -1:
 				fprintf(stderr,
 					" Directory has no parent entry\n");
@@ -101,7 +99,7 @@ static int renameit(dos_name_t *dosname,
 			}
 
 			wipeEntry(movedEntry);
-			
+
 			/* free the old parent, allocate the new one. */
 			oldDir = movedEntry->Dir;
 			*movedEntry = *targetEntry;
@@ -174,7 +172,7 @@ static int rename_directory(direntry_t *entry, MainParam_t *mp)
 	ret = rename_file(entry, mp);
 	if(ret & ERROR_ONE)
 		return ret;
-	
+
 	return ret;
 }
 
@@ -220,7 +218,7 @@ static void usage(int ret)
 	fprintf(stderr,
 		"Usage: %s [-vV] [-D clash_option] file targetfile\n", progname);
 	fprintf(stderr,
-		"       %s [-vV] [-D clash_option] file [files...] target_directory\n", 
+		"       %s [-vV] [-D clash_option] file [files...] target_directory\n",
 		progname);
 	exit(ret);
 }
@@ -270,7 +268,7 @@ void mmove(int argc, char **argv, int oldsyntax)
 	if (argc - optind < 2)
 		usage(1);
 
-	init_mp(&arg.mp);		
+	init_mp(&arg.mp);
 	arg.mp.arg = (void *) &arg;
 	arg.mp.openflags = O_RDWR;
 
@@ -312,10 +310,12 @@ void mmove(int argc, char **argv, int oldsyntax)
 	}
 
 
-	arg.mp.longname = longname;
+	arg.mp.longname.data = longname;
+	arg.mp.longname.len = sizeof(longname);
 	longname[0]='\0';
 
-	arg.mp.shortname = shortname;
+	arg.mp.shortname.data = shortname;
+	arg.mp.shortname.len = sizeof(shortname);
 	shortname[0]='\0';
 
 	exit(main_loop(&arg.mp, argv + optind, argc - optind - 1));
