@@ -19,7 +19,6 @@
 #define DONT_NEED_WAIT
 
 #include "sysincludes.h"
-#include "msdos.h"
 #include "mtools.h"
 #include "mainloop.h"
 #include "fsP.h"
@@ -67,7 +66,7 @@ static void set_offset(hsc *h, unsigned long offset,
 void setBeginEnd(struct partition *partTable,
 		 uint32_t begin, uint32_t end,
 		 uint16_t iheads, uint16_t isectors,
-		 int activate, uint8_t type, int fat_bits)
+		 int activate, uint8_t type, unsigned int fat_bits)
 {
 	uint8_t heads, sectors;
 
@@ -85,7 +84,7 @@ void setBeginEnd(struct partition *partTable,
 		exit(1);
 	}
 	sectors=(uint8_t) isectors;
-	
+
 	set_offset(&partTable->start, begin, heads, sectors);
 	set_offset(&partTable->end, end-1, heads, sectors);
 	set_dword(partTable->start_sect, begin);
@@ -253,14 +252,6 @@ static void usage(int ret)
 	exit(ret);
 }
 
-static void checkTotalSectors(unsigned long tot_sectors) {
-	if(tot_sectors > UINT32_MAX) {
-		fprintf(stderr, "Too many total sectors %ld\n",
-			tot_sectors);
-		exit(1);
-	}
-}
-
 void mpartition(int argc, char **argv, int dummy UNUSEDP) NORETURN;
 void mpartition(int argc, char **argv, int dummy UNUSEDP)
 {
@@ -305,7 +296,7 @@ void mpartition(int argc, char **argv, int dummy UNUSEDP)
 	char errmsg[2100];
 	char *bootSector=0;
 	struct partition *tpartition;
-	
+
 	argtracks = 0;
 	argheads = 0;
 	argsectors = 0;
@@ -387,7 +378,7 @@ void mpartition(int argc, char **argv, int dummy UNUSEDP)
 				break;
 			case 'l':
 				size_set = 1;
-				length = strtoui(optarg, &endptr, 0);
+				length = parseSize(optarg);
 				break;
 
 			default:
@@ -433,7 +424,7 @@ void mpartition(int argc, char **argv, int dummy UNUSEDP)
 #ifdef USING_NEW_VOLD
 		strcpy(name, getVoldName(dev, name));
 #endif
-		Stream = OpenImage(&used_dev, dev, name, mode, errmsg, 
+		Stream = OpenImage(&used_dev, dev, name, mode, errmsg,
 				   open2flags | SKIP_PARTITION | ALWAYS_GET_GEOMETRY,
 				   mode, NULL, NULL, NULL);
 
@@ -448,9 +439,9 @@ void mpartition(int argc, char **argv, int dummy UNUSEDP)
 		}
 
 		tot_sectors = used_dev.tot_sectors;
-		
+
 		/* read the partition table */
-		if (READS(Stream, (char *) buf, 0, 512) != 512 && !initialize){
+		if (PREADS(Stream, (char *) buf, 0, 512) != 512 && !initialize){
 #ifdef HAVE_SNPRINTF
 			snprintf(errmsg, sizeof(errmsg)-1,
 				"Error reading from '%s', wrong parameters?",
@@ -557,7 +548,6 @@ void mpartition(int argc, char **argv, int dummy UNUSEDP)
 
 	if(!used_dev.sectors && !used_dev.heads) {
 		if(tot_sectors) {
-			checkTotalSectors(tot_sectors);
 			setsize0((uint32_t)tot_sectors,&dummy2,&used_dev.heads,
 				 &used_dev.sectors);
 		} else {
@@ -574,9 +564,7 @@ void mpartition(int argc, char **argv, int dummy UNUSEDP)
 	if(create) {
 		unsigned int overlap;
 		if(!end_set && !size_set && tot_sectors) {
-			checkTotalSectors(tot_sectors);
-			end = (uint32_t) tot_sectors -
-				(uint32_t) tot_sectors % sec_per_cyl;
+			end = tot_sectors - tot_sectors % sec_per_cyl;
 			end_set = 1;
 		}
 
@@ -602,7 +590,7 @@ void mpartition(int argc, char **argv, int dummy UNUSEDP)
 			fprintf(stderr, "Begin larger than end\n");
 			exit(1);
 		}
-		
+
 		/* Check whether new partition doesn't overlap with
 		 * any of those already in place */
 		if((overlap=findOverlap(partTable, 4, begin, end))) {
@@ -615,7 +603,7 @@ void mpartition(int argc, char **argv, int dummy UNUSEDP)
 		setBeginEnd(tpartition, begin, end,
 			    used_dev.heads, used_dev.sectors,
 			    !has_activated, type,
-			    dev->fat_bits);
+			    abs(dev->fat_bits));
 	}
 
 	if(activate) {
@@ -682,7 +670,7 @@ void mpartition(int argc, char **argv, int dummy UNUSEDP)
 		/* write data back to the disk */
 		if(verbose>=2)
 			print_sector("Writing sector", buf, 512);
-		if (WRITES(Stream, (char *) buf, 0, 512) != 512) {
+		if (PWRITES(Stream, (char *) buf, 0, 512) != 512) {
 			fprintf(stderr,"Error writing partition table");
 			exit(1);
 		}
